// This is -*- C++ -*-
// $Id: test_matrix.cc,v 1.3 1998/09/14 04:47:42 trow Exp $

/* test_matrix.cc
 *
 * Copyright (C) 1998 EMC Capital Management, Inc.
 *
 * Developed by Jon Trowbridge <trow@emccta.com>.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */

#include <iostream.h>
#include <time.h>
#include <math.h>
#include "Random.h"
#include "matrix.h"

void check(double* a, double* ainv, int N)
{
  matrix_invert(a,N);
  bool problem = false;
  for(int i=0; i<N*N && !problem; ++i)
    if (fabs(a[i] - ainv[i]) > 1e-8)
      problem = true;
  cout << N << "x" << N << " inversion ";
  cout << (problem ? "failed!" : "succeeded") << endl;
}

main()
{
  double     x1[4] = {1, 2,
                      3, 4};
  double inv_x1[4] = { -2,    1,
		      1.5, -0.5};

  double     x2[9] = {1, 2, 3, 
		      3, 2, 1,
		      1, 3, 2};
  double inv_x2[9] = { 0.0833333333,  0.4166666666, -0.3333333333, 
		      -0.4166666666, -0.0833333333,  0.6666666666, 
		       0.5833333333, -0.0833333333, -0.3333333333};

  cout << "Checking fixed examples..." << endl;
  check(x1,inv_x1,2);
  check(x2,inv_x2,3);

  Random rnd;

  cout << endl << "Checking random examples... " << endl;
  const int M = 100;
  double m1[M*M], m2[M*M];
  for(int i=2; i<=M; ++i) {
    do {
      rnd.random_normal(m1,i*i);
      memcpy(m2,m1,sizeof(double)*i*i);
    } while (! matrix_invert(m2,i));
    check(m2,m1,i);
  }

  cout << endl << "Running inversion benchmark..." << endl;
  
  const int N = 40;
  const int TRIALS = 10000;
  double m[N*N];

  time_t t1, t2;

  time(&t1);
  for(int i=0; i<TRIALS; ++i) {
    rnd.random_normal(m, N*N);
    matrix_invert(m,N);
  }
  time(&t2);

  cout << "Generated & inverted ";
  cout << TRIALS << " " << N << "x" << N << " matrices in ";
  cout << (t2-t1) << " seconds." << endl; 
  cout << "Inversions/second = " << TRIALS/(double)(t2-t1) << endl;
}




// $Id: test_matrix.cc,v 1.3 1998/09/14 04:47:42 trow Exp $
