// This is -*- C++ -*-
// $Id: ConfInt.h,v 1.2 1998/09/14 04:48:37 trow Exp $

/* 
 * ConfInt.h
 *
 * Copyright (C) 1998 EMC Capital Management, Inc.
 *
 * Developed by Jon Trowbridge <trow@emccta.com>.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */

#ifndef _INC_ConfInt_H
#define _INC_ConfInt_H

#include <nan.h>
#include <math.h>

class ConfInt {
public:
  ConfInt() { lo_ = est_ = hi_ = conf_ = NAN; }

  ConfInt(const ConfInt& e) : lo_(e.lo_), est_(e.est_), hi_(e.hi_),
    conf_(e.conf_)
    { }

  void set(double l, double e, double h) {
    if (l <= e && e <= h) {
      lo_ = l;
      est_ = e;
      hi_ = h;
    }
  }

  void set_by_error(double x, double err) {
    est_ = x;
    err = fabs(err);
    lo_ = x-err;
    hi_ = x+err;
  }

  void set_confidence(double c) { conf_ = c; }

  double low() const { return lo_; }
  double est() const { return est_; }
  double high() const { return hi_; }
  double width() const { return hi_-lo_; }

  double confidence() const { return conf_; }

  bool contains(double x) const { return lo_ <= x && x <= hi_; }

private:
  double lo_, est_, hi_, conf_;
};

#endif // _INC_ConfInt_H

// $Id: ConfInt.h,v 1.2 1998/09/14 04:48:37 trow Exp $
