// This is -*- C++ -*-

/* 
 * DataType.cc
 *
 * Copyright (C) 1998 EMC Capital Management, Inc.
 *
 * Developed by Havoc Pennington <hp@pobox.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */


#include "DataType.h"

bool 
RealType::identify (const string & text) const
{
  // make a copy to work with
  string s = text;

  size_t p = s.find_first_of("eE",0);

  // if we have an exponent, slice it off and check it
  if (p != s.npos) {
    string expo = s.substr(p);
    s.erase(p);

    if (expo.size() < 2)
      return false;
    
    // make sure that both the 1e10, 1e+10 and 1e-10 type notation
    // is handled correctly
    if (expo[1] == '-' || expo[1] == '+') {
      if (expo.size() < 3)
	return false;
    } else if (!isdigit(expo[1]))
      return false;

    // make sure that the exponent contains only digits
    if (expo.find_first_not_of("0123456789",2) != expo.npos)
      return false;
  }

  // the empty string is not a float
  if (s.size() == 0)
    return false;

  // check the mantissa
  bool seenDecPt = false;
  for(size_t i=0; i<s.size(); ++i) {
    char c = s[i];
    if (!(isdigit(c) ||
	  (c == '-' && i == 0) || // only allow negative in first place
	  (c == '.' && !seenDecPt))) // only one dec pt per mantissa
      return false;
    if (c == '.')
      seenDecPt = true;
  }
  return true;
}


