// This is -*- C++ -*-
// $Id: bootstrap.cc,v 1.5 1998/09/14 04:48:37 trow Exp $

/* bootstrap.cc
 *
 * Copyright (C) 1998 EMC Capital Management, Inc.
 *
 * Developed by Jon Trowbridge <trow@emccta.com>.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */

#include <iostream.h>
#include "bootstrap.h"

DataSet
bootstrap_mean(size_t N, const DataSet& ds)
{
  DataSet boot;
  boot.reserve(N);

  Random rnd;
  int* ind = new int[ds.size()];

  for(size_t i=0; i<N; ++i) {
    double sum=0;
    rnd.random_integer(0, ds.size()-1, ind, ds.size());
    for(size_t j=0; j<ds.size(); ++j) 
      sum += ds.data(ind[j]);
    boot.add(sum / ds.size());
  }

  delete [] ind;

  return boot;
}

DataSet
bootstrap_median(size_t N, const DataSet& ds)
{
  DataSet boot;
  boot.reserve(N);

  const size_t dsN = ds.size();

  Random rnd;
  int* ind = new int[dsN];
  int* count = new int[dsN];

  for(size_t i=0; i<N; ++i) {

    // precalc random data indices
    rnd.random_integer(0,dsN-1, ind, dsN);

    // clear element counts, map random indices into count buffer
    memset(count, 0, dsN * sizeof(int));
    for(size_t j=0; j<dsN; ++j)
      ++count[ind[j]];

    int run=0, target=(dsN-1)/2;
    size_t j=0;
    for(;j<dsN && run <= target; ++j) 
      run += count[j];
    
    double med;
    if (dsN % 2 == 0 /* even */) {
      size_t jj=j-1;
      if(run - target == 1)
	while (count[++jj] == 0);
      med = (ds.sorted_data(j-1) + ds.sorted_data(jj))/2;
    } else {
      med = ds.sorted_data(j-1);
    }
    boot.add(med);
  }

  delete [] ind;
  delete [] count;

  return boot;
}

DataSet
bootstrap_sdev(size_t N, const DataSet& ds)
{
  DataSet boot;
  boot.reserve(N);

  Random rnd;
  int* ind = new int[ds.size()];
  
  for(size_t i=0; i<N; ++i) {
    rnd.random_integer(0, ds.size()-1, ind, ds.size());
    double sum=ds.data(ind[0]), sumsq=0;
    for(size_t j=1; j<ds.size(); ++j) {
      double x = ds.data(ind[j]);
      sum += x;
      sumsq += (x-sum/(j+1))*(x-(sum-x)/j);
    }
    boot.add(sqrt(sumsq/ds.size()));
  }

  delete [] ind;

  return boot;
}


// $Id: bootstrap.cc,v 1.5 1998/09/14 04:48:37 trow Exp $
