// This is -*- C++ -*-
// $Id: normal.cc,v 1.4 1998/09/14 04:48:38 trow Exp $

/* normal.cc
 *
 * Copyright (C) 1998 EMC Capital Management, Inc.
 *
 * Developed by Jon Trowbridge <trow@emccta.com>.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */

#include <math.h>
#include "statfn.h"
#include "normal.h"

ConfInt
ci_mean(const DataSet& ds, double c)
{
  double m = ds.mean();
  double se = ds.sdevs() / sqrt(ds.size());
  double t_val = inv_students_cdf((1+c)/2,ds.size()-1);

  ConfInt ci;
  ci.set_confidence(c);
  ci.set_by_error(m, se*t_val);
  return ci;
}

ConfInt
ci_diff_mean(const DataSet& ds1, const DataSet& ds2, double c)
{
  double m1 = ds1.mean();
  double m2 = ds2.mean();
  double pv = ds1.pooled_var(ds2);
  double t_val = inv_students_cdf((1+c)/2, ds1.size()+ds2.size()-2);

  ConfInt ci;
  ci.set_confidence(c);
  ci.set_by_error(m1-m2, sqrt(pv*(1.0/ds1.size() + 1.0/ds2.size())) * t_val);
  return ci;
}


// $Id: normal.cc,v 1.4 1998/09/14 04:48:38 trow Exp $
