// This is -*- C++ -*-
// $Id: regression.h,v 1.4 1998/09/14 04:48:38 trow Exp $

/* 
 * Regression.h
 *
 * Copyright (C) 1998 EMC Capital Management, Inc.
 *
 * Developed by Jon Trowbridge <trow@emccta.com>.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */

#ifndef _INC_REGRESSION_H
#define _INC_REGRESSION_H

#include "DataSet.h"

class LinearRegression {
public:
  LinearRegression() : slope_(0), intercept_(0), corr_(0) { }
  LinearRegression(const DataSet& indep_var, const DataSet& dep_var) {
    model(indep_var, dep_var);
  }

  void model(const DataSet& indep_var, const DataSet& dep_var);
  DataSet residuals() { return residuals_; }

  double slope() const { return slope_; }
  double intercept() const { return intercept_; }
  double correlation() const { return corr_; }
  double predict(double x) const { return slope_ * x + intercept_; }

private:
  double slope_, intercept_, corr_;
  DataSet residuals_;
};

class PolynomialRegression {
public:
  PolynomialRegression() : degree_(-1), coeff_(0) { }
  PolynomialRegression(const DataSet& indep_var, const DataSet& dep_var,
		       int degree) : degree_(-1), coeff_(0) {
    model(indep_var, dep_var, degree);
  }
  ~PolynomialRegression() { delete [] coeff_; }

  void model(const DataSet& indep_var, const DataSet& dep_var,
	     int degree);
  DataSet residuals() { return residuals_; }

  int degree() const { return degree_; }
  double coefficient(int i) const { return i<0 || i>degree_ ? 0 : coeff_[i]; }
  double predict(double x) const {
    int i = degree_;
    double y = coeff_[i];
    while (i > 0) y = y*x + coeff_[--i];
    return y;
  }

private:
  int degree_;
  double* coeff_;
  DataSet residuals_;
};

#endif // _INC_REGRESSION_H

// $Id: regression.h,v 1.4 1998/09/14 04:48:38 trow Exp $
