// This is -*- C++ -*-
// $Id: runs.cc,v 1.3 1998/09/14 04:48:38 trow Exp $

/* runs.cc
 *
 * Copyright (C) 1998 EMC Capital Management, Inc.
 *
 * Developed by Jon Trowbridge <trow@emccta.com>.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */

#include <math.h>
#include "statfn.h"
#include "runs.h"

// runs_mean() and runs_sdev() are based on asymptotic properties of the
// distribution of runs.  For small samples, these will be inaccurate.
// Eventually I need to add look-up tables of tail probabilities for
// sample sizes up to about 20 or so.

double
RunsCount::runs_mean() const
{
  return size_ ? 1+2.0*t_*f_/size_ : 0;
}

double
RunsCount::runs_sdev() const
{
  if (size_ == 0 || t_ == size_ || f_ == size_)
    return 0;
  double tf = t_*f_;
  double N = size_;
  return sqrt(2*tf*(2*tf-N)/(N*N*(N-1)));
}

double
RunsCount::p_value() const
{
  return 2*normal_cdf(fabs(z()))-1;
}


void
RunsCount::add(bool x)
{
  if (size_ == 0 || last_ != x) ++runs_;
  ++size_;
  x ? ++t_ : ++f_;
  last_ = x;
}

////////////////////////////////////////////////////////////////

double
RunsUpDown::runs_mean() const
{
  return size_ > 2 ? (2.0*size_-1)/3.0 : (size_ ? 1 : 0);
}

double
RunsUpDown::runs_sdev() const
{
  return size_ >= 2 ? sqrt((16.0*size_-29.0)/90.0) : 0;
}

double
RunsUpDown::p_value() const
{
  return 2*normal_cdf(fabs(z()))-1;
}

void
RunsUpDown::add(double x)
{
  if (size_ == 0) {
    ++runs_;
    constant_ = true;
  } else if (constant_) {
    if (x != last_) {
      last_higher_ = (x > last_);
      constant_ = false;
    }
  } else {
    if (last_higher_ ? (x < last_) : (x > last_)) {
      last_higher_ = !last_higher_;
      ++runs_;
    }
  }
  last_ = x;
  ++size_;
}



// $Id: runs.cc,v 1.3 1998/09/14 04:48:38 trow Exp $
